@@ -3,25 +3,20 @@
 use strict;
 use warnings;
 
+use IO::Async::Loop;
+
 use Tickit;
 
 use Tickit::Widget::Static;
 use Tickit::Widget::Entry;
 
-use Tickit::Widget::Box;
 use Tickit::Widget::VBox;
 
 my $vbox = Tickit::Widget::VBox->new( spacing => 1 );
 
 $vbox->add( Tickit::Widget::Static->new( text => "Enter some text here:" ) );
 
-$vbox->add( Tickit::Widget::Box->new(
-   h_border => 2,
-   v_border => 1,
-   bg => 'blue',
-   child =>
-      ( my $entry = Tickit::Widget::Entry->new ),
-) );
+$vbox->add( my $entry = Tickit::Widget::Entry->new );
 
 $vbox->add( my $label = Tickit::Widget::Static->new( text => "" ) );
 
@@ -30,7 +25,10 @@ $entry->set_on_enter( sub {
    $_[0]->set_text( "" );
 } );
 
+my $loop = IO::Async::Loop->new;
+
 my $tickit = Tickit->new();
+$loop->add( $tickit );
 
 $tickit->set_root_widget( $vbox );
 
