@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
 
 package Tickit::Widget::Entry;
 
@@ -9,9 +9,9 @@ use strict;
 use warnings;
 use base qw( Tickit::Widget );
 
-our $VERSION = '0.17_003';
+our $VERSION = '0.03';
 
-use Tickit::Utils qw( textwidth chars2cols cols2chars substrwidth );
+use Text::CharWidth qw( mbswidth );
 
 # Positions in this code can get complicated. The following conventions apply:
 #   $pos_ch  = a position in CHaracters within a Unicode string (length, substr,..)
@@ -26,8 +26,12 @@ C<Tickit::Widget::Entry> - a widget for entering text
 
  use Tickit;
  use Tickit::Widget::Entry;
+ use IO::Async::Loop;
+ 
+ my $loop = IO::Async::Loop->new;
  
  my $tickit = Tickit->new;
+ $loop->add( $tickit );
  
  my $entry = Tickit::Widget::Entry->new(
     on_enter => sub {
@@ -79,7 +83,7 @@ Delete one character forwards
 
 Delete one word forwards
 
-=item * End or Ctrl-E
+=item * End
 
 Move the cursor to the end of the input line
 
@@ -87,14 +91,10 @@ Move the cursor to the end of the input line
 
 Accept a line of input by running the C<on_enter> action
 
-=item * Home or Ctrl-A
+=item * Home
 
 Move the cursor to the beginning of the input line
 
-=item * Insert
-
-Toggle between overwrite and insert mode
-
 =item * Left
 
 Move the cursor one character left
@@ -138,12 +138,7 @@ sub new
    my $textlen = length $self->{text};
    $self->{pos_ch} = $textlen if $self->{pos_ch} > $textlen;
 
-   $self->{scrolloffs_co} = 0;
-   $self->{overwrite} = 0;
-
    $self->{keybindings} = {
-      'C-a' => "key_beginning_of_line",
-      'C-e' => "key_end_of_line",
       'C-k' => "key_delete_line",
       'C-u' => "key_backward_delete_line",
       'C-w' => "key_backward_delete_word",
@@ -157,7 +152,6 @@ sub new
       'End'       => "key_end_of_line",
       'Enter'     => "key_enter_line",
       'Home'      => "key_beginning_of_line",
-      'Insert'    => "key_overwrite_mode",
       'Left'      => "key_backward_char",
       'C-Left'    => "key_backward_word",
       'Right'     => "key_forward_char",
@@ -166,9 +160,6 @@ sub new
 
    $self->set_on_enter( $params{on_enter} ) if defined $params{on_enter};
 
-   $self->{more_markers} = [ "<..", "..>" ];
-   $self->{more_pen} = Tickit::Pen->new( fg => "cyan" );
-
    return $self;
 }
 
@@ -180,199 +171,32 @@ sub char2col
    my $self = shift;
    my ( $ch ) = @_;
 
-   return scalar chars2cols $self->{text}, $ch;
+   return mbswidth( substr $self->{text}, 0, $ch );
 }
 
-sub pretext_width
-{
-   my $self = shift;
-
-   return 0 if $self->{scrolloffs_co} == 0;
-   return textwidth( $self->{more_markers}[0] );
-}
-
-sub pretext_render
-{
-   my $self = shift;
-   my ( $win ) = @_;
-
-   $win->print( $self->{more_markers}[0], $self->{more_pen} );
-}
-
-sub posttext_width
-{
-   my $self = shift;
-
-   return 0 if textwidth( $self->text ) <= $self->{scrolloffs_co} + $self->window->cols;
-   return textwidth( $self->{more_markers}[1] );
-}
-
-sub posttext_render
-{
-   my $self = shift;
-   my ( $win ) = @_;
-
-   $win->print( $self->{more_markers}[1], $self->{more_pen} );
-}
-
-use constant CLEAR_BEFORE_RENDER => 0;
-
 sub render
 {
    my $self = shift;
-   my %args = @_;
 
    my $win = $self->window or return;
-   $win->is_visible or return;
-   my $rect = $args{rect};
 
-   if( $rect->top == 0 ) {
-      $win->goto( 0, 0 );
+   $win->goto( 0, 0 );
 
-      my $width = $win->cols;
+   my $text = $self->{text};
+   $win->print( $text );
 
-      my $text = substrwidth( $self->text, $self->{scrolloffs_co}, $width );
-      my $textlen_co = textwidth $text;
-
-      my $pretext_width  = $self->pretext_width;
-      my $posttext_width = $self->posttext_width;
-
-      $self->pretext_render( $win ) if $pretext_width;
-
-      $win->print( substrwidth( $text, $pretext_width, $width - $pretext_width - $posttext_width ) );
-      if( $textlen_co < $width - $posttext_width ) {
-         $win->erasech( $width - $posttext_width - $textlen_co );
-      }
-
-      $self->posttext_render( $win ) if $posttext_width;
-   }
-
-   foreach my $line ( $rect->top + 1 .. $win->lines - 1 ) {
-      $win->clearline( $line );
-   }
+   $win->erasech( $win->cols - length $text );
 
    $self->reposition_cursor;
 }
 
-sub _recalculate_scroll
-{
-   my $self = shift;
-   my ( $pos_ch ) = @_;
-
-   my $pos_co = $self->char2col( $pos_ch );
-   my $off_co = $self->{scrolloffs_co};
-
-   my $pos_x = $pos_co - $off_co;
-
-   my $width = $self->window->cols;
-   my $halfwidth = int( $width / 2 );
-
-   # Try to keep the cursor within 5 columns of the window edge
-   while( $pos_x < 5 and $off_co >= 5 ) {
-      $off_co -= $halfwidth;
-      $off_co = 0 if $off_co < 0;
-      $pos_x = $pos_co - $off_co;
-   }
-   while( $pos_x > ( $width - 5 ) ) {
-      $off_co += $halfwidth;
-      $pos_x = $pos_co - $off_co;
-   }
-
-   return $off_co if $off_co != $self->{scrolloffs_co};
-   return undef;
-}
-
 sub reposition_cursor
 {
    my $self = shift;
-   my ( $pos_ch ) = @_;
 
    my $win = $self->window or return;
 
-   $self->{pos_ch} = $pos_ch if defined $pos_ch;
-
-   my $new_scrolloffs = $self->_recalculate_scroll( $self->{pos_ch} );
-   if( defined $new_scrolloffs ) {
-      $self->{scrolloffs_co} = $new_scrolloffs;
-      $self->redraw;
-   }
-
-   my $pos_x = $self->char2col( $self->{pos_ch} ) - $self->{scrolloffs_co};
-
-   $win->focus( 0, $pos_x );
-}
-
-sub _text_spliced
-{
-   my $self = shift;
-   my ( $pos_ch, $deleted, $inserted, $at_end ) = @_;
-
-   my $win = $self->window;
-   my $width = $win->cols;
-
-   my $insertedlen_co = textwidth $inserted;
-   my $deletedlen_co  = textwidth $deleted;
-
-   my $delta_co = $insertedlen_co - $deletedlen_co;
-
-   my $pos_co = $self->char2col( $pos_ch );
-   my $pos_x  = $pos_co - $self->{scrolloffs_co};
-
-   # Don't bother at all if the affected range is scrolled off the right
-   return if $pos_x >= $width;
-
-   if( $pos_x < 0 ) {
-      die "TODO: text_splice before window - what to do??\n";
-   }
-
-   my $need_reprint = 0;
-
-   if( $delta_co > 0 and !$at_end or
-       $delta_co < 0 ) {
-      $win->scrollrect( 0, $pos_x, 1, $win->cols - $pos_x, 0, $delta_co ) or
-         $need_reprint = 1;
-   }
-
-   if( $need_reprint ) {
-      # ICH/DCH failed; we'll have to reprint the entire rest of the line from
-      # here
-      my $right_co = $self->{scrolloffs_co} + $width;
-
-      $win->goto( 0, $pos_x );
-      $win->print( substrwidth( $self->text, $pos_co, $right_co - $pos_co ) );
-
-      my $trailing_blank_co = $right_co - length($self->text);
-      $win->erasech( $trailing_blank_co ) if $trailing_blank_co > 0;
-
-      $win->goto( 0, $pos_x );
-      return;
-   }
-
-   if( $insertedlen_co > 0 ) {
-      my $right_co = $self->{scrolloffs_co} + $width;
-
-      my $end_co = $pos_co + $insertedlen_co;
-
-      if( $end_co > $right_co ) {
-         substrwidth( $inserted, $right_co - $pos_co, "" );
-      }
-
-      $win->goto( 0, $pos_x );
-      $win->print( $inserted );
-   }
-
-   if( $delta_co < 0 and $self->{scrolloffs_co} + $width < textwidth $self->text ) {
-      my $marker = $self->{more_markers}[1];
-      my $damagewidth = -$delta_co + textwidth $marker;
-
-      my $rhs_x  = $width - $damagewidth;
-      my $rhs_co = $rhs_x + $self->{scrolloffs_co};
-
-      $win->goto( 0, $rhs_x );
-      $win->print( substrwidth( $self->text, $rhs_co, $damagewidth ) );
-
-      $win->goto( 0, $pos_x + $insertedlen_co );
-   }
+   $win->focus( 0, $self->char2col( $self->{pos_ch} ) );
 }
 
 sub on_key
@@ -381,8 +205,7 @@ sub on_key
    my ( $type, $str, $key ) = @_;
 
    if( $type eq "key" and my $code = $self->{keybindings}{$str} ) {
-      $self->$code( $str, $key );
-      return 1;
+      return $self->$code( $str, $key );
    }
    if( $type eq "text" ) {
       $self->on_text( $str );
@@ -397,18 +220,7 @@ sub on_text
    my $self = shift;
    my ( $text ) = @_;
 
-   $self->text_splice( $self->{pos_ch}, $self->{overwrite} ? 1 : 0, $text );
-}
-
-sub on_mouse
-{
-   my $self = shift;
-   my ( $ev, $button, $line, $col ) = @_;
-
-   return unless $ev eq "press" and $button == 1;
-
-   my $pos_ch = scalar cols2chars $self->{text}, $col + $self->{scrolloffs_co};
-   $self->set_position( $pos_ch );
+   $self->text_insert( $text, $self->{pos_ch} );
 }
 
 =head1 ACCESSORS
@@ -466,7 +278,9 @@ sub set_position
    $pos_ch = 0 if $pos_ch < 0;
    $pos_ch = length $self->{text} if $pos_ch > length $self->{text};
 
-   $self->reposition_cursor( $pos_ch );
+   $self->{pos_ch} = $pos_ch;
+
+   $self->reposition_cursor;
 
    if( my $win = $self->window ) {
       $win->restore;
@@ -561,9 +375,9 @@ sub text_insert
    $self->text_splice( $pos_ch, 0, $text );
 }
 
-=head2 $deleted = $entry->text_delete( $pos_ch, $len_ch )
+=head2 $entry->text_delete( $pos_ch, $len_ch )
 
-Delete the given section of text. Returns the deleted text.
+Delete the given section of text.
 
 =cut
 
@@ -572,13 +386,12 @@ sub text_delete
    my $self = shift;
    my ( $pos_ch, $len_ch ) = @_;
 
-   return $self->text_splice( $pos_ch, $len_ch, "" );
+   $self->text_splice( $pos_ch, $len_ch, "" );
 }
 
-=head2 $deleted = $entry->text_splice( $pos_ch, $len_ch, $text )
+=head2 $entry->text_splice( $pos_ch, $len_ch, $text )
 
-Replace the given section of text with the given replacement. Returns the
-text deleted from the section.
+Replace the given section of text with the given replacement.
 
 =cut
 
@@ -587,38 +400,45 @@ sub text_splice
    my $self = shift;
    my ( $pos_ch, $len_ch, $text ) = @_;
 
+   my $win = $self->window;
+
+   my $textlen_co = mbswidth $text;
    my $textlen_ch = length($text);
 
+   my $deletedlen_co = $len_ch ? mbswidth( substr $self->{text}, $pos_ch, $len_ch ) : 0;
+
    my $delta_ch = $textlen_ch - $len_ch;
+   my $delta_co = $textlen_co - $deletedlen_co;
 
-   my $at_end = ( $pos_ch == length $self->{text} );
+   if( $pos_ch != $self->{pos_ch} ) {
+      $win->goto( 0, $self->char2col( $pos_ch ) );
+   }
 
-   my $deleted = substr( $self->{text}, $pos_ch, $len_ch, $text );
+   if( $delta_co > 0 ) {
+      $win->insertch( $delta_co ) if $pos_ch < length $self->{text};
+   }
+   elsif( $delta_co < 0 ) {
+      $win->deletech( -$delta_co );
+   }
 
-   my $new_pos_ch;
+   $win->print( $text ) if $textlen_ch;
+   substr( $self->{text}, $pos_ch, $len_ch ) = $text;
 
    if( $self->{pos_ch} >= $pos_ch + $len_ch ) {
       # Cursor after splice; move to suit
-      $new_pos_ch = $self->position + $delta_ch;
+      $self->{pos_ch} += $delta_ch;
    }
-   elsif( $self->{pos_ch} >= $pos_ch ) {
-      # Cursor within splice; move to end
-      $new_pos_ch = $pos_ch + $textlen_ch;
+   elsif( $self->{pos_ch} > $pos_ch ) {
+      # Cursor within splice; move to start
+      $self->{pos_ch} = $pos_ch;
    }
    # else { ignore }
 
-   # No point incrementally updating as we'll have to scroll anyway
-   unless( defined $new_pos_ch and defined $self->_recalculate_scroll( $new_pos_ch ) ) {
-      $self->_text_spliced( $pos_ch, $deleted, $text, $at_end );
-   }
-
-   $self->reposition_cursor( $new_pos_ch ) if defined $new_pos_ch and $new_pos_ch != $self->{pos_ch};
+   $self->reposition_cursor;
 
    if( $pos_ch + $textlen_ch != $self->{pos_ch} ) {
-      $self->window->restore;
+      $win->restore;
    }
-
-   return $deleted;
 }
 
 =head2 $pos = $entry->find_bow_forward( $initial, $else )
@@ -806,13 +626,6 @@ sub key_forward_word
    $self->set_position( $bow );
 }
 
-sub key_overwrite_mode
-{
-   my $self = shift;
-
-   $self->{overwrite} = !$self->{overwrite};
-}
-
 =head1 TODO
 
 =over 4
@@ -823,14 +636,23 @@ Try to find a nice way to allow loaded plugins, possibly per-instance if not
 just globally or per-class. See how many of these TODO items can be done using
 plugins.
 
+=item * Sideways scrolling
+
+Scroll text left or right to fit the cursor within the visible range. Display
+"more" markers at either end as appropriate.
+
 =item * More readline behaviours
 
-History. Isearch. History replay. Transpose. Transcase. Yank ring. Numeric
-prefixes.
+Insert/overwrite. History. Isearch. History replay. Transpose. Transcase. Yank
+ring. Numeric prefixes.
 
 =item * Visual selection behaviour
 
-Shift-movement, or vim-style. Mouse.
+Shift-movement, or vim-style.
+
+=item * Mouse support
+
+Click to position cursor. Drag to start a selection.
 
 =back
 
